# Copyright (C) 2019-2021 all contributors <meta@public-inbox.org>
# License: AGPL-3.0+ <https://www.gnu.org/licenses/agpl-3.0.txt>
local $ENV{PI_TEST_VERSION} = 1;
require './t/indexlevels-mirror.t';
